#include "viewer.h"

using namespace std;
using namespace qglviewer;

//************************************************************
// Implementation of Viewer
//************************************************************
Viewer::Viewer(QWidget* parent,const char* name)
  : QGLViewer(parent,name),
    showSilhouettes(false),
    showShadowQuads(false),
    showShadows(false)
{
}
void
Viewer::loadModel(const char* fileName)
{
  OpenMesh::IO::Options ropt;
  if (!OpenMesh::IO::read_mesh(mesh,fileName,ropt))
  {
    cerr<<"Error loading mesh from file "<<fileName<<endl;
    return;
  }
  cout<<"OpenMesh imported ["<<fileName<<"]"<<endl;
  mesh.request_face_normals();
  mesh.request_vertex_normals();
  mesh.update_normals();
  // Compute bounding box
  bbox.clear();
  for (Mesh::VertexIter v=mesh.vertices_begin();v!=mesh.vertices_end();++v)
  {
    bbox.addPoint(mesh.point(v));
  }  
  setSceneCenter(bbox.center());
  setSceneRadius(bbox.sizeMax()/2.0f);
  showEntireScene();
}
void
Viewer::init()
{
  light = new qglviewer::ManipulatedFrame();
  light->setPosition(bbox.center());
  connect(light,SIGNAL(modified()),this,SLOT(updateGL()));
  setMouseTracking(true);
  glClearColor(0.498039f,0.498039f,0.7490f,1.0f);
}
void
Viewer::preDraw()
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
  // GL_PROJECTION matrix
  camera()->loadProjectionMatrix();
  // GL_MODELVIEW matrix
  camera()->loadModelViewMatrix();
}
void
Viewer::draw()
{
  glPushAttrib(GL_ALL_ATTRIB_BITS);
  //////////////////////////////////////////////////////////////
  // Draw the light
  //////////////////////////////////////////////////////////////
  Vec L = light->position();
  float Lw[4] = { L[0],L[1],L[2],1.0f };
  glLightfv(GL_LIGHT0,GL_POSITION,Lw);

  // Code pour la Q1
  //       .
  //       .
  //       .
  //       .
  // Code pour la Q1
  
  //////////////////////////////////////////////////////////////
  // Draw the model
  //////////////////////////////////////////////////////////////
  if (showShadows)
  {
    // Code pour la Q5
    //       .
    //       .
    //       .
    //       .
    // Code pour la Q5
  }
  else
  {
    glEnable(GL_LIGHTING);
    glColor3f(1.0f,1.0f,1.0f);
    renderModel();
  }
  //////////////////////////////////////////////////////////////
  // Draw the silhouette edges
  //////////////////////////////////////////////////////////////
  if (showSilhouettes)
  {
    glDisable(GL_LIGHTING);
    glDepthRange(0.0f,0.999f);
    glColor3f(1.0f,0.0f,0.0f);
    glLineWidth(3.0f);
    renderSilhouettes();
    glDepthRange(0.0f,1.0f);
  }
  //////////////////////////////////////////////////////////////
  // Draw the shadow quads
  //////////////////////////////////////////////////////////////
  if (showShadowQuads)
  {
    glDisable(GL_LIGHTING);
    glEnable(GL_BLEND);
    glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_ALPHA);
    static const float color[3] = { 1.0f,1.0f,0.0f };
    static const float a = 0.5f;
    glColor4f(color[0]*a,color[1]*a,color[2]*a,a);
    renderShadowQuads();
    glDisable(GL_BLEND);
  }
  //
  glPopAttrib();
}
void
Viewer::renderModel() const
{
  // Code pour la Q2
  //       .
  //       .
  //       .
  //       .
  // Code pour la Q2
}
void
Viewer::renderSilhouettes() const
{
  Vec L = light->position();
  // Code pour la Q3
  //       .
  //       .
  //       .
  //       .
  // Code pour la Q3
}
void
Viewer::renderShadowQuads() const
{
  Vec L = light->position();
  // Code pour la Q4
  //       .
  //       .
  //       .
  //       .
  // Code pour la Q4
}
void
Viewer::keyPressEvent(QKeyEvent* e)
{
  switch (e->key())
  {
  case Key_R: restoreFromFile();updateGL();break;
  case Key_E: showSilhouettes = !showSilhouettes;updateGL();break;
  case Key_Q: showShadowQuads = !showShadowQuads;updateGL();break;
  case Key_A: showShadows = !showShadows;updateGL();break;
  default:
    QGLViewer::keyPressEvent(e);
  }
}
